@@ -56,11 +56,8 @@ static bool             playBackWasInit = false;
 static AudioUnit        theUnit;
 static AudioFilePlayer* thePlayer = NULL;
 static CDPlayerCompletionProc   completionProc = NULL;
-static pthread_mutex_t  apiMutex;
-static pthread_t        callbackThread;
-static pthread_mutex_t  callbackMutex;
-static volatile  int    runCallBackThread;
-static int              initMutex = SDL_TRUE;
+static SDL_mutex       *apiMutex = NULL;
+static SDL_sem         *callbackSem;
 static SDL_CD*          theCDROM;
 
 //
@@ -69,36 +66,26 @@ static SDL_CD*          theCDROM;
 
 #pragma mark -- Prototypes --
 
-OSStatus CheckInit ();
+static OSStatus CheckInit ();
 
-OSStatus MatchAUFormats (AudioUnit theUnit, UInt32 theInputBus);
+static void     FilePlayNotificationHandler (void* inRefCon, OSStatus inStatus);
 
-void     FilePlayNotificationHandler (void* inRefCon, OSStatus inStatus);
-
-void*    RunCallBackThread (void* inRefCon);
+static int      RunCallBackThread (void* inRefCon);
 
 
 #pragma mark -- Public Functions --
 
 void     Lock ()
 {
-    if (initMutex) {
-    
-        pthread_mutexattr_t attr;
-        
-        pthread_mutexattr_init (&attr);
-        pthread_mutex_init (&apiMutex, &attr);
-        pthread_mutexattr_destroy (&attr);
-        
-        initMutex = SDL_FALSE;
+    if (!apiMutex) {
+        apiMutex = SDL_CreateMutex();
     }
-    
-    pthread_mutex_lock (&apiMutex);
+    SDL_mutexP(apiMutex);
 }
 
 void     Unlock ()
 {
-    pthread_mutex_unlock (&apiMutex);
+    SDL_mutexV(apiMutex);
 }
 
 //
@@ -114,9 +101,7 @@ int DetectAudioCDVolumes(FSVolumeRefNum *volumes, int numVolumes)
     for (volumeIndex = 1; result == noErr || result != nsvErr; volumeIndex++)
     {
         FSVolumeRefNum  actualVolume;
-        HFSUniStr255    volumeName;
         FSVolumeInfo    volumeInfo;
-        FSRef           rootDirectory;
         
         memset (&volumeInfo, 0, sizeof(volumeInfo));
         
@@ -125,13 +110,13 @@ int DetectAudioCDVolumes(FSVolumeRefNum *volumes, int numVolumes)
                                   &actualVolume,
                                   kFSVolInfoFSInfo,
                                   &volumeInfo,
-                                  &volumeName,
-                                  &rootDirectory); 
+                                  NULL,
+                                  NULL); 
          
         if (result == noErr)
         {
             if (volumeInfo.filesystemID == kAudioCDFilesystemID) // It's an audio CD
-            {                
+            {
                 if (volumes != NULL && cdVolumeCount < numVolumes)
                     volumes[cdVolumeCount] = actualVolume;
             
@@ -357,7 +342,7 @@ int ListTrackFiles (FSVolumeRefNum theVolume, FSRef *trackFiles, int numTracks)
                                  
     if (result != noErr) {
         SDL_SetError ("ListTrackFiles: FSGetVolumeInfo returned %d", result);
-        goto bail;
+        return result;
     }
 
     result = FSOpenIterator (&rootDirectory, kFSIterateFlat, &iterator);
@@ -372,15 +357,19 @@ int ListTrackFiles (FSVolumeRefNum theVolume, FSRef *trackFiles, int numTracks)
                 name = CFStringCreateWithCharacters (NULL, nameStr.unicode, nameStr.length);
                 
                 // Look for .aiff extension
-                if (CFStringHasSuffix (name, CFSTR(".aiff"))) {
+                if (CFStringHasSuffix (name, CFSTR(".aiff")) ||
+                    CFStringHasSuffix (name, CFSTR(".cdda"))) {
                     
                     // Extract the track id from the filename
                     int trackID = 0, i = 0;
-                    while (nameStr.unicode[i] >= '0' && nameStr.unicode[i] <= '9') {
-                        trackID = 10 * trackID +(nameStr.unicode[i] - '0');
-                        i++;
+                    while (i < nameStr.length && !isdigit(nameStr.unicode[i])) {
+                        ++i;
                     }
-                    
+                    while (i < nameStr.length && isdigit(nameStr.unicode[i])) {
+                        trackID = 10 * trackID +(nameStr.unicode[i] - '0');
+                        ++i;
+                    }
+
                     #if DEBUG_CDROM
                     printf("Found AIFF for track %d: '%s'\n", trackID, 
                     CFStringGetCStringPtr (name, CFStringGetSystemEncoding()));
@@ -400,9 +389,7 @@ int ListTrackFiles (FSVolumeRefNum theVolume, FSRef *trackFiles, int numTracks)
         FSCloseIterator (iterator);
     }
     
-    result = 0;
-  bail:   
-    return result;
+    return 0;
 }
 
 //
@@ -434,7 +421,7 @@ int LoadFile (const FSRef *ref, int startFrame, int stopFrame)
             throw (-3);
         }
             
-        thePlayer->SetDestination(theUnit, 0);
+        thePlayer->SetDestination(theUnit);
         
         if (startFrame >= 0)
             thePlayer->SetStartFrame (startFrame);
@@ -561,6 +548,9 @@ void SetCompletionProc (CDPlayerCompletionProc proc, SDL_CD *cdrom)
     thePlayer->SetNotifier (FilePlayNotificationHandler, cdrom);
 }
 
+//
+//  GetCurrentFrame
+//
 
 int GetCurrentFrame ()
 {    
@@ -577,26 +567,18 @@ int GetCurrentFrame ()
 
 #pragma mark -- Private Functions --
 
-OSStatus CheckInit ()
+static OSStatus CheckInit ()
 {    
     if (playBackWasInit)
         return 0;
     
     OSStatus result = noErr;
     
-        
-    // Create the callback mutex
-    pthread_mutexattr_t attr;
-    pthread_mutexattr_init (&attr);
-    pthread_mutex_init (&callbackMutex, &attr);
-    pthread_mutexattr_destroy (&attr);
-    pthread_mutex_lock (&callbackMutex);
-        
+    // Create the callback semaphore
+    callbackSem = SDL_CreateSemaphore(0);
+
     // Start callback thread
-    pthread_attr_t attr1;
-    pthread_attr_init (&attr1);        
-    pthread_create (&callbackThread, &attr1, RunCallBackThread, NULL);
-    pthread_attr_destroy (&attr1);
+    SDL_CreateThread(RunCallBackThread, NULL);
 
     try {
         ComponentDescription desc;
@@ -621,16 +603,6 @@ OSStatus CheckInit ()
             THROW_RESULT("CheckInit: AudioUnitInitialize")
         
                     
-        // In this case we first want to get the output format of the OutputUnit
-        // Then we set that as the input format. Why?
-        // So that only a single conversion process is done
-        // when SetDestination is called it will get the input format of the
-        // unit its supplying data to. This defaults to 44.1K, stereo, so if
-        // the device is not that, then we lose a possibly rendering of data
-        
-        result = MatchAUFormats (theUnit, 0);
-            THROW_RESULT("CheckInit: MatchAUFormats")
-    
         playBackWasInit = true;
     }
     catch (...)
@@ -641,35 +613,12 @@ OSStatus CheckInit ()
     return 0;
 }
 
-
-OSStatus MatchAUFormats (AudioUnit theUnit, UInt32 theInputBus)
-{
-    AudioStreamBasicDescription theDesc;
-    UInt32 size = sizeof (theDesc);
-    OSStatus result = AudioUnitGetProperty (theUnit,
-                                            kAudioUnitProperty_StreamFormat,
-                                            kAudioUnitScope_Output,
-                                            0,
-                                            &theDesc,
-                                            &size);
-        THROW_RESULT("MatchAUFormats: AudioUnitGetProperty")
-
-    result = AudioUnitSetProperty (theUnit,
-                                   kAudioUnitProperty_StreamFormat,
-                                   kAudioUnitScope_Input,
-                                   theInputBus,
-                                   &theDesc,
-                                   size);
-    
-    return result;
-}
-
-void FilePlayNotificationHandler(void * inRefCon, OSStatus inStatus)
+static void FilePlayNotificationHandler(void * inRefCon, OSStatus inStatus)
 {
     if (inStatus == kAudioFilePlay_FileIsFinished) {
     
         // notify non-CA thread to perform the callback
-        pthread_mutex_unlock (&callbackMutex);
+        SDL_SemPost(callbackSem);
         
     } else if (inStatus == kAudioFilePlayErr_FilePlayUnderrun) {
     
@@ -683,13 +632,11 @@ void FilePlayNotificationHandler(void * inRefCon, OSStatus inStatus)
     }
 }
 
-void* RunCallBackThread (void *param)
+static int RunCallBackThread (void *param)
 {
-    runCallBackThread = 1;
+    for (;;) {
     
-    while (runCallBackThread) {
-    
-        pthread_mutex_lock (&callbackMutex);
+	SDL_SemWait(callbackSem);
 
         if (completionProc && theCDROM) {
             #if DEBUG_CDROM
@@ -703,13 +650,11 @@ void* RunCallBackThread (void *param)
         }
     }
     
-    runCallBackThread = -1;
-    
     #if DEBUG_CDROM
     printf ("thread dying now...\n");
     #endif
     
-    return NULL;
+    return 0;
 }
 
 }; // extern "C"
