@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
+    Copyright (C) 1997-2004 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_events.c,v 1.9 2002/05/29 23:31:33 slouken Exp $";
+ "@(#) $Id: SDL_events.c,v 1.12 2005/08/21 06:18:54 slouken Exp $";
 #endif
 
 /* General event handling code for SDL */
@@ -91,7 +91,6 @@ void SDL_Unlock_EventThread(void)
 
 static int SDL_GobbleEvents(void *unused)
 {
-	SDL_SetTimerThreaded(2);
 	event_thread = SDL_ThreadID();
 	while ( SDL_EventQ.active ) {
 		SDL_VideoDevice *video = current_video;
@@ -114,7 +113,7 @@ static int SDL_GobbleEvents(void *unused)
 
 		/* Give up the CPU for the rest of our timeslice */
 		SDL_EventLock.safe = 1;
-		if( SDL_timer_running ) {
+		if ( SDL_timer_running ) {
 			SDL_ThreadedTimerCheck();
 		}
 		SDL_Delay(1);
@@ -162,6 +161,8 @@ static int SDL_StartEventThread(Uint32 flags)
 		}
 		SDL_EventLock.safe = 0;
 
+		/* The event thread will handle timers too */
+		SDL_SetTimerThreaded(2);
 		SDL_EventThread = SDL_CreateThread(SDL_GobbleEvents, NULL);
 		if ( SDL_EventThread == NULL ) {
 			return(-1);
@@ -195,6 +196,12 @@ void SDL_StopEventLoop(void)
 	/* Halt the event thread, if running */
 	SDL_StopEventThread();
 
+	/* Shutdown event handlers */
+	SDL_AppActiveQuit();
+	SDL_KeyboardQuit();
+	SDL_MouseQuit();
+	SDL_QuitQuit();
+
 	/* Clean out EventQ */
 	SDL_EventQ.head = 0;
 	SDL_EventQ.tail = 0;
