@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_lowvideo.h,v 1.6 2004/01/04 16:49:24 slouken Exp $";
+ "@(#) $Id: SDL_lowvideo.h,v 1.7 2004/07/18 19:36:06 slouken Exp $";
 #endif
 
 #ifndef _SDL_lowvideo_h
@@ -49,14 +49,6 @@ struct SDL_PrivateVideoData {
 	/* A completely clear cursor */
 	WMcursor *BlankCursor;
 
-	/* Mouse state variables */
-	uint32 last_buttons;
-	BPoint last_point;
-
-	/* Keyboard state variables */
-	int key_flip;
-	struct key_info keyinfo[2];
-	
 	SDL_Overlay *overlay;
 };
 /* Old variable names */
@@ -65,10 +57,6 @@ struct SDL_PrivateVideoData {
 #define SDL_nummodes	(_this->hidden->SDL_nummodes)
 #define SDL_modelist	(_this->hidden->SDL_modelist)
 #define SDL_BlankCursor	(_this->hidden->BlankCursor)
-#define last_buttons	(_this->hidden->last_buttons)
-#define last_point	(_this->hidden->last_point)
-#define key_flip	(_this->hidden->key_flip)
-#define keyinfo		(_this->hidden->keyinfo)
 #define current_overlay (_this->hidden->overlay)
 
 #endif /* _SDL_lowvideo_h */
