@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_ph_video.c,v 1.24 2004/02/17 16:19:29 slouken Exp $";
+ "@(#) $Id: SDL_ph_video.c,v 1.26 2004/07/18 19:46:37 slouken Exp $";
 #endif
 
 #include <stdlib.h>
@@ -47,6 +47,7 @@ static char rcsid =
 #include "SDL_ph_events_c.h"
 #include "SDL_ph_mouse_c.h"
 #include "SDL_ph_wm_c.h"
+#include "SDL_ph_gl.h"
 #include "SDL_phyuv_c.h"
 #include "blank_cursor.h"
 
@@ -56,14 +57,6 @@ static int  ph_SetColors(_THIS, int firstcolor, int ncolors, SDL_Color *colors);
 static void ph_VideoQuit(_THIS);
 static void ph_DeleteDevice(SDL_VideoDevice *device);
 
-#ifdef HAVE_OPENGL
-static void  ph_GL_SwapBuffers(_THIS);
-static int   ph_GL_GetAttribute(_THIS, SDL_GLattr attrib, int* value);
-static int   ph_GL_LoadLibrary(_THIS, const char* path);
-static void* ph_GL_GetProcAddress(_THIS, const char* proc);
-static int   ph_GL_MakeCurrent(_THIS);
-#endif /* HAVE_OPENGL */
-
 static int phstatus=-1;
 
 static int ph_Available(void)
@@ -83,27 +76,28 @@ static int ph_Available(void)
     return 1;
 }
 
-static SDL_VideoDevice *ph_CreateDevice(int devindex)
+static SDL_VideoDevice* ph_CreateDevice(int devindex)
 {
-    SDL_VideoDevice *device;
+    SDL_VideoDevice* device;
 
     /* Initialize all variables that we clean on shutdown */
     device = (SDL_VideoDevice *)malloc(sizeof(SDL_VideoDevice));
-    if (device) {
+    if (device)
+    {
         memset(device, 0, (sizeof *device));
-        device->hidden = (struct SDL_PrivateVideoData *)
-                malloc((sizeof *device->hidden));
+        device->hidden = (struct SDL_PrivateVideoData*)malloc((sizeof *device->hidden));
         device->gl_data = NULL;
     }
-    if ((device == NULL) || (device->hidden == NULL)) {
+    if ((device == NULL) || (device->hidden == NULL))
+    {
         SDL_OutOfMemory();
         ph_DeleteDevice(device);
-        return(0);
+        return NULL;
     }
     memset(device->hidden, 0, (sizeof *device->hidden));
 
     /* Set the driver flags */
-    device->handles_any_size = 1; /* JB not true for fullscreen */
+    device->handles_any_size = 1;
 
     /* Set the function pointers */
     device->CreateYUVOverlay = ph_CreateYUVOverlay;
@@ -113,7 +107,7 @@ static SDL_VideoDevice *ph_CreateDevice(int devindex)
     device->ToggleFullScreen = ph_ToggleFullScreen;
     device->UpdateMouse = ph_UpdateMouse;
     device->SetColors = ph_SetColors;
-    device->UpdateRects = NULL;         /* set up in ph_SetupUpdateFunction */
+    device->UpdateRects = NULL;                        /* set up in ph_SetupUpdateFunction */
     device->VideoQuit = ph_VideoQuit;
     device->AllocHWSurface = ph_AllocHWSurface;
     device->CheckHWBlit = ph_CheckHWBlit;
@@ -254,6 +248,10 @@ static int ph_SetupWindow(_THIS, int w, int h, int flags)
     }
     else
     {
+        PtSetArg(&args[nargs++], Pt_ARG_WINDOW_MANAGED_FLAGS, Pt_FALSE, Ph_WM_FFRONT | Ph_WM_CONSWITCH);
+        PtSetArg(&args[nargs++], Pt_ARG_WINDOW_STATE, Pt_FALSE, Ph_WM_STATE_ISFRONT);
+        PtSetArg(&args[nargs++], Pt_ARG_WINDOW_STATE, Pt_TRUE, Ph_WM_STATE_ISALTKEY);
+
         if ((flags & SDL_HWSURFACE) == SDL_HWSURFACE)
         {
             PtSetArg(&args[nargs++], Pt_ARG_BASIC_FLAGS, Pt_TRUE, Pt_BASIC_PREVENT_FILL);
@@ -371,6 +369,7 @@ static int ph_VideoInit(_THIS, SDL_PixelFormat* vformat)
 
 #ifdef HAVE_OPENGL
     oglctx=NULL;
+    oglbuffers=NULL;
     oglflags=0;
     oglbpp=0;
 #endif /* HAVE_OPENGL */
@@ -450,20 +449,15 @@ static int ph_VideoInit(_THIS, SDL_PixelFormat* vformat)
     OCImage.FrameData1 = NULL;
     videomode_emulatemode = 0;
     
-    this->info.video_mem=hwcaps.currently_available_video_ram/1024;
     this->info.wm_available = 1;
-    this->info.hw_available = 1;
-    this->info.blit_fill = 1;
-    this->info.blit_hw = 1;
-    this->info.blit_hw_A = 0;
-    this->info.blit_hw_CC = 1;
+
+    ph_UpdateHWInfo(this);
     
     return 0;
 }
 
 static SDL_Surface* ph_SetVideoMode(_THIS, SDL_Surface *current, int width, int height, int bpp, Uint32 flags)
 {
-    PgHWCaps_t hwcaps;
     const struct ColourMasks* mask;
 
     /* Lock the event thread, in multi-threading environments */
@@ -558,6 +552,7 @@ static SDL_Surface* ph_SetVideoMode(_THIS, SDL_Surface *current, int width, int
     /* Must call at least once for setup image planes */
     if (ph_SetupUpdateFunction(this, current, current->flags)==-1)
     {
+        /* Error string was filled in the ph_SetupUpdateFunction() */
         return NULL;
     }
 
@@ -573,12 +568,7 @@ static SDL_Surface* ph_SetVideoMode(_THIS, SDL_Surface *current, int width, int
 
     visualbpp=bpp;
 
-    if (PgGetGraphicsHWCaps(&hwcaps) < 0)
-    {
-        SDL_SetError("ph_SetVideoMode(): GetGraphicsHWCaps function failed !\n");
-        return NULL;
-    }
-    this->info.video_mem=hwcaps.currently_available_video_ram/1024;
+    ph_UpdateHWInfo(this);
 
     SDL_Unlock_EventThread();
 
@@ -667,57 +657,3 @@ static int ph_SetColors(_THIS, int firstcolor, int ncolors, SDL_Color *colors)
     return 1;
 }
 
-#ifdef HAVE_OPENGL
-
-static void ph_GL_SwapBuffers(_THIS)
-{
-    PgSetRegion(PtWidgetRid(window));
-    PdOpenGLContextSwapBuffers(oglctx);
-}
-
-static int ph_GL_GetAttribute(_THIS, SDL_GLattr attrib, int* value)
-{
-    switch (attrib)
-    {
-        case SDL_GL_DOUBLEBUFFER:
-             *value=this->gl_config.double_buffer;
-             break;
-        case SDL_GL_STENCIL_SIZE:
-             *value=this->gl_config.stencil_size;
-             break;
-        case SDL_GL_DEPTH_SIZE:
-             *value=this->gl_config.depth_size;
-             break;
-        default:
-             *value=0;
-             return(-1);
-    }
-    return 0;
-}
-
-static int ph_GL_LoadLibrary(_THIS, const char* path)
-{
-   /* if code compiled with HAVE_OPENGL, that mean that library already linked */
-   this->gl_config.driver_loaded = 1;
-
-   return 0;
-}
-
-static void* ph_GL_GetProcAddress(_THIS, const char* proc)
-{
-   return NULL;
-}
-
-static int ph_GL_MakeCurrent(_THIS)
-{
-    PgSetRegion(PtWidgetRid(window));
-
-    if (oglctx!=NULL)
-    {
-        PhDCSetCurrent(oglctx);
-    }
-
-    return 0;
-}
-
-#endif /* HAVE_OPENGL */
