@@ -17,12 +17,12 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_gsevents_c.h,v 1.1 2001/06/16 03:17:45 hercules Exp $";
+ "@(#) $Id: SDL_gsevents_c.h,v 1.3 2002/03/06 11:23:06 slouken Exp $";
 #endif
 
 #include "SDL_gsvideo.h"
