@@ -17,12 +17,12 @@
 	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 	Sam Lantinga
-	slouken@devolution.com
+	slouken@libsdl.org
 */
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_gsvideo.c,v 1.3 2001/08/01 08:41:45 slouken Exp $";
+ "@(#) $Id: SDL_gsvideo.c,v 1.5 2002/03/06 11:23:06 slouken Exp $";
 #endif
 
 /* Framebuffer console based SDL video driver implementation.
