@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997-2004 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -21,7 +21,7 @@
 */
 
 /*
-     File added by Alan Buckley (alan_baa@hotmail.com) for RISCOS compatability
+     File added by Alan Buckley (alan_baa@hotmail.com) for RISC OS compatability
 	 27 March 2003
 
      Implements keyboard setup, event pump and keyboard and mouse polling
@@ -43,10 +43,10 @@
 #include "kernel.h"
 #include "swis.h"
 
-/* The translation table from a RISCOS internal key numbers to a SDL keysym */
+/* The translation table from a RISC OS internal key numbers to a SDL keysym */
 static SDLKey RO_keymap[SDLK_LAST];
 
-/* RISCOS Key codes */
+/* RISC OS Key codes */
 #define ROKEY_SHIFT 0
 #define ROKEY_CTRL  1
 #define ROKEY_ALT   2
@@ -71,24 +71,28 @@ static char RO_pressed[ROKEYBD_ARRAYSIZE];
 
 static SDL_keysym *TranslateKey(int intkey, SDL_keysym *keysym, int pressed);
 
-
 void RISCOS_PollMouse(_THIS);
 void RISCOS_PollKeyboard();
 
 void RISCOS_PollMouseHelper(_THIS, int fullscreen);
 
+#ifdef DISABLE_THREADS
 extern void DRenderer_FillBuffers();
 
 /* Timer running function */
 extern void RISCOS_CheckTimer();
 
+#endif
+
 void FULLSCREEN_PumpEvents(_THIS)
 {
     /* Current implementation requires keyboard and mouse polling */
 	RISCOS_PollKeyboard();
 	RISCOS_PollMouse(this);
-	DRenderer_FillBuffers();
+#ifdef DISABLE_THREADS
+//	DRenderer_FillBuffers();
 	if (SDL_timer_running) RISCOS_CheckTimer();
+#endif
 }
 
 
@@ -144,7 +148,7 @@ void RISCOS_InitOSKeymap(_THIS)
   RO_keymap[53] = SDLK_u;
   RO_keymap[54] = SDLK_o;
   RO_keymap[55] = SDLK_p;
-  RO_keymap[56] = SDLK_LEFTPAREN;
+  RO_keymap[56] = SDLK_LEFTBRACKET;
   RO_keymap[57] = SDLK_UP;
   RO_keymap[58] = SDLK_KP_PLUS;
   RO_keymap[59] = SDLK_KP_MINUS;
@@ -173,7 +177,7 @@ void RISCOS_InitOSKeymap(_THIS)
   RO_keymap[85] = SDLK_n;
   RO_keymap[86] = SDLK_l;
   RO_keymap[87] = SDLK_SEMICOLON;
-  RO_keymap[88] = SDLK_RIGHTPAREN;
+  RO_keymap[88] = SDLK_RIGHTBRACKET;
   RO_keymap[89] = SDLK_DELETE;
   RO_keymap[90] = SDLK_KP_MINUS;
   RO_keymap[91] = SDLK_KP_MULTIPLY;
@@ -258,9 +262,11 @@ void RISCOS_PollMouseHelper(_THIS, int fullscreen)
        Sint16 new_x = regs.r[0]; /* Initialy get as OS units */
        Sint16 new_y = regs.r[1];
 
-/* Discard mouse events until the let go of the mouse after starting */
-       if (starting && regs.r[2] != 0) return;
-       else starting = 0;
+       /* Discard mouse events until they let go of the mouse after starting */
+       if (starting && regs.r[2] != 0)
+         return;
+       else
+         starting = 0;
 
        if (new_x != last_x || new_y != last_y || last_buttons != regs.r[2])
        {
@@ -356,7 +362,7 @@ void RISCOS_PollKeyboard()
 	{
 		which_key = (_kernel_osbyte(121, which_key, 0) & 0xFF);
 	    if (which_key != ROKEY_NONE)
-		{
+	    {
 		    switch(which_key)
 		    {
 		    /* Skip over mouse keys */
@@ -370,6 +376,10 @@ void RISCOS_PollKeyboard()
 			case 71: case 24: case 87: case 40:
 			    break;
 
+            /* Ignore break as it can be latched on */
+            case 44:
+                break;
+
 			default:
 				RO_pressed[which_key] += 2;
 				break;
@@ -426,11 +436,11 @@ static SDL_keysym *TranslateKey(int intkey, SDL_keysym *keysym, int pressed)
 
 		state = (_kernel_osbyte(202, 0, 255) & 0xFF);
 
-		/*TODO: better key to char mapping */
+		/*TODO: Take into account other keyboard layouts */
 
-		ch = keysym->sym; /* This should handle most keys */
+		ch = keysym->sym; /* This should handle most unshifted keys */
 
-        if (intkey < 9)
+        if (intkey < 9 || ch == SDLK_UNKNOWN)
         {
            ch = 0;
 
@@ -438,15 +448,95 @@ static SDL_keysym *TranslateKey(int intkey, SDL_keysym *keysym, int pressed)
 		{
 			ch = ch & 31;
 
-		} else if ((state & 16) == 0) /* Caps lock is on */
+		} else 
 		{
-			if ((state & 128) == 0) /* Shift Enable off */
+			int topOfKey = 0;
+            if (state & 8) /* Shift on */
 			{
-				ch = toupper(ch);
+				topOfKey = 1;
 			}
-		} else if (state & 8) /* Shift on */
-		{
-			ch = toupper(ch);
+
+			if ((state & 16) == 0) /* Caps lock is on */
+			{
+			   if (ch >= SDLK_a && ch <= SDLK_z)
+			   {
+ 				  if ((state & 128) == 0) /* Shift Enable off */
+				  {
+				     /* All letter become upper case */
+				 	 topOfKey = 1;
+				  } else
+				  {
+				     /* Shift+Letters gives lower case */
+				     topOfKey = 1 - topOfKey;
+				  }
+		       }
+			}
+
+			if (topOfKey)
+			{
+				/* Key produced with shift held down */
+
+				/* Letters just give upper case version */
+				if (ch >= SDLK_a && ch <= SDLK_z) ch = toupper(ch);
+				else
+				{
+					switch(ch)
+					{
+					case SDLK_HASH:   ch = '~'; break;
+					case SDLK_QUOTE:  ch = '@'; break;
+					case SDLK_COMMA:  ch = '<'; break;
+					case SDLK_MINUS:  ch = '_'; break;
+					case SDLK_PERIOD: ch = '>'; break;
+					case SDLK_SLASH:  ch = '?'; break;
+
+					case SDLK_0: ch = ')'; break;
+					case SDLK_1: ch = '!'; break;
+					case SDLK_2: ch = '"'; break;
+					case SDLK_3: ch = ''; break;
+					case SDLK_4: ch = '$'; break;
+					case SDLK_5: ch = '%'; break;
+					case SDLK_6: ch = '^'; break;
+					case SDLK_7: ch = '&'; break;
+					case SDLK_8: ch = '*'; break;
+					case SDLK_9: ch = '('; break;
+
+					case SDLK_SEMICOLON:    ch = ':'; break;
+					case SDLK_EQUALS:       ch = '+'; break;
+					case SDLK_LEFTBRACKET:  ch = '{'; break;
+					case SDLK_BACKSLASH:    ch = '|'; break;
+					case SDLK_RIGHTBRACKET: ch = '}'; break;
+					case SDLK_BACKQUOTE:    ch = ''; break;
+
+					default:
+						ch = 0; /* Map to zero character if we don't understand it */
+						break;
+					}
+				}
+
+			} else if (ch > 126)
+			{
+				/* SDL key code < 126 map directly onto their Unicode equivalents */
+				/* Keypad 0 to 9 maps to numeric equivalent */
+				if (ch >= SDLK_KP0 && ch <= SDLK_KP9) ch = ch - SDLK_KP0 + '0';
+				else
+				{
+					/* Following switch maps other keys that produce an Ascii value */
+					switch(ch)
+					{
+					case SDLK_KP_PERIOD:   ch = '.'; break;
+					case SDLK_KP_DIVIDE:   ch = '/'; break;
+					case SDLK_KP_MULTIPLY: ch = '*'; break;
+					case SDLK_KP_MINUS:    ch = '-'; break;
+					case SDLK_KP_PLUS:     ch = '+'; break;
+					case SDLK_KP_EQUALS:   ch = '='; break;
+
+					default:
+						/* If we don't know what it is set the Unicode to 0 */
+						ch = 0;
+						break;
+					}
+				}
+			}			
 		}
 				
 		keysym->unicode = ch;
