@@ -17,12 +17,12 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_sysmouse.c,v 1.3 2001/04/29 23:00:03 hercules Exp $";
+ "@(#) $Id: SDL_sysmouse.c,v 1.7 2002/10/11 08:09:26 slouken Exp $";
 #endif
 
 #include <stdlib.h>
@@ -95,6 +95,8 @@ static void memxor(Uint8 *dst, Uint8 *src1, Uint8 *src2, int len)
 void WIN_FreeWMCursor(_THIS, WMcursor *cursor)
 {
 #ifndef USE_STATIC_CURSOR
+	if ( cursor->curs == GetCursor() )
+		SetCursor(NULL);
 	if ( cursor->curs != NULL )
 		DestroyCursor(cursor->curs);
 	if ( cursor->ands != NULL )
@@ -211,12 +213,7 @@ int WIN_ShowWMCursor(_THIS, WMcursor *cursor)
 
 void WIN_WarpWMCursor(_THIS, Uint16 x, Uint16 y)
 {
-	POINT pt;
-
 	if ( DDRAW_FULLSCREEN() ) {
-		x += (this->screen->offset % this->screen->pitch) /
-		      this->screen->format->BytesPerPixel;
-		y += (this->screen->offset / this->screen->pitch);
 		SDL_PrivateMouseMotion(0, 0, x, y);
 	} else if ( mouse_relative) {
 		/*	RJR: March 28, 2000
@@ -224,6 +221,7 @@ void WIN_WarpWMCursor(_THIS, Uint16 x, Uint16 y)
 			mouse hidden and grabbed */
 		SDL_PrivateMouseMotion(0, 0, x, y);
 	} else {
+		POINT pt;
 		pt.x = x;
 		pt.y = y;
 		ClientToScreen(SDL_Window, &pt);
