@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
+    Copyright (C) 1997-2004 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_dx5video.c,v 1.21 2002/10/11 07:55:22 slouken Exp $";
+ "@(#) $Id: SDL_dx5video.c,v 1.28 2004/11/12 23:22:08 slouken Exp $";
 #endif
 
 #include <stdio.h>
@@ -71,6 +71,7 @@ HRESULT (WINAPI *DInputCreate)(HINSTANCE hinst, DWORD dwVersion, LPDIRECTINPUT *
 /* This is the rect EnumModes2 uses */
 struct DX5EnumRect {
 	SDL_Rect r;
+	int refreshRate;
 	struct DX5EnumRect* next;
 };
 static struct DX5EnumRect *enumlists[NUM_MODELISTS];
@@ -650,8 +651,10 @@ static HRESULT WINAPI EnumModes2(DDSURFACEDESC *desc, VOID *udata)
 	struct DX5EnumRect *enumrect;
 #if defined(NONAMELESSUNION)
 	int bpp = desc->ddpfPixelFormat.u1.dwRGBBitCount;
+	int refreshRate = desc->u2.dwRefreshRate;
 #else
 	int bpp = desc->ddpfPixelFormat.dwRGBBitCount;
+	int refreshRate = desc->dwRefreshRate;
 #endif
 
 	switch (bpp)  {
@@ -660,22 +663,37 @@ static HRESULT WINAPI EnumModes2(DDSURFACEDESC *desc, VOID *udata)
 		case 24:
 		case 32:
 			bpp /= 8; --bpp;
+			if ( enumlists[bpp] &&
+			     enumlists[bpp]->r.w == (Uint16)desc->dwWidth &&
+			     enumlists[bpp]->r.h == (Uint16)desc->dwHeight ) {
+				if ( refreshRate > enumlists[bpp]->refreshRate &&
+				     refreshRate <= 85 /* safe value? */ ) {
+					enumlists[bpp]->refreshRate = refreshRate;
+#ifdef DDRAW_DEBUG
+ fprintf(stderr, "New refresh rate for %d bpp: %dx%d at %d Hz\n", (bpp+1)*8, (int)desc->dwWidth, (int)desc->dwHeight, refreshRate);
+#endif
+				}
+				break;
+			}
 			++SDL_nummodes[bpp];
 			enumrect = (struct DX5EnumRect*)malloc(sizeof(struct DX5EnumRect));
 			if ( !enumrect ) {
 				SDL_OutOfMemory();
 				return(DDENUMRET_CANCEL);
 			}
+			enumrect->refreshRate = refreshRate;
 			enumrect->r.x = 0;
 			enumrect->r.y = 0;
 			enumrect->r.w = (Uint16)desc->dwWidth;
 			enumrect->r.h = (Uint16)desc->dwHeight;
 			enumrect->next = enumlists[bpp];
 			enumlists[bpp] = enumrect;
+#ifdef DDRAW_DEBUG
+ fprintf(stderr, "New mode for %d bpp: %dx%d at %d Hz\n", (bpp+1)*8, (int)desc->dwWidth, (int)desc->dwHeight, refreshRate);
+#endif
 			break;
 	}
 
-
 	return(DDENUMRET_OK);
 }
 
@@ -912,7 +930,7 @@ int DX5_VideoInit(_THIS, SDL_PixelFormat *vformat)
 	for ( i=0; i<NUM_MODELISTS; ++i )
 		enumlists[i] = NULL;
 
-	result = IDirectDraw2_EnumDisplayModes(ddraw2,0,NULL,this,EnumModes2);
+	result = IDirectDraw2_EnumDisplayModes(ddraw2,DDEDM_REFRESHRATES,NULL,this,EnumModes2);
 	if ( result != DD_OK ) {
 		SetDDerror("DirectDraw2::EnumDisplayModes", result);
 		return(-1);
@@ -926,7 +944,7 @@ int DX5_VideoInit(_THIS, SDL_PixelFormat *vformat)
 			return(-1);
 		}
 		for ( j = 0, rect = enumlists[i]; rect; ++j, rect = rect->next ) {
-			SDL_modelist[i][j]=(SDL_Rect *)rect;
+			SDL_modelist[i][j] = &rect->r;
 		}
 		SDL_modelist[i][j] = NULL;
 	}
@@ -985,17 +1003,14 @@ SDL_Surface *DX5_SetVideoMode(_THIS, SDL_Surface *current,
 			(WS_OVERLAPPED|WS_CAPTION|WS_SYSMENU|WS_MINIMIZEBOX);
 	const DWORD resizestyle =
 			(WS_THICKFRAME|WS_MAXIMIZEBOX);
+	int windowX, windowY;
 	DDSURFACEDESC ddsd;
 	LPDIRECTDRAWSURFACE  dd_surface1;
 	LPDIRECTDRAWSURFACE3 dd_surface3;
-	BOOL was_visible;
 
 #ifdef DDRAW_DEBUG
  fprintf(stderr, "Setting %dx%dx%d video mode\n", width, height, bpp);
 #endif
-	/* See whether or not we should center the window */
-	was_visible = IsWindowVisible(SDL_Window);
-
 	/* Clean up any previous DirectDraw surfaces */
 	if ( current->hwdata ) {
 		this->FreeHWSurface(this, current);
@@ -1116,31 +1131,57 @@ SDL_Surface *DX5_SetVideoMode(_THIS, SDL_Surface *current,
 			if (IsZoomed(SDL_Window)) style |= WS_MAXIMIZE;
 #endif
 		}
-		SetWindowLong(SDL_Window, GWL_STYLE, style);
+
+		/* DJM: Don't piss of anyone who has setup his own window */
+		if ( SDL_windowid == NULL )
+			SetWindowLong(SDL_Window, GWL_STYLE, style);
 
 		/* Resize the window (copied from SDL WinDIB driver) */
 		if ( SDL_windowid == NULL ) {
 			HWND top;
 			UINT swp_flags;
+			const char *window = getenv("SDL_VIDEO_WINDOW_POS");
+			const char *center = getenv("SDL_VIDEO_CENTERED");
+
+			if ( !SDL_windowX && !SDL_windowY ) {
+				if ( window ) {
+					if ( sscanf(window, "%d,%d", &x, &y) == 2 ) {
+						SDL_windowX = x;
+						SDL_windowY = y;
+					}
+					if ( strcmp(window, "center") == 0 ) {
+						center = window;
+						window = NULL;
+					}
+				}
+			}
+			swp_flags = (SWP_NOCOPYBITS | SWP_SHOWWINDOW);
 
 			SDL_resizing = 1;
-			bounds.top    = 0;
-			bounds.bottom = video->h;
-			bounds.left   = 0;
-			bounds.right  = video->w;
+			bounds.left = SDL_windowX;
+			bounds.top = SDL_windowY;
+			bounds.right = SDL_windowX+video->w;
+			bounds.bottom = SDL_windowY+video->h;
 			AdjustWindowRectEx(&bounds, GetWindowLong(SDL_Window, GWL_STYLE), FALSE, 0);
 			width = bounds.right-bounds.left;
 			height = bounds.bottom-bounds.top;
-			x = (GetSystemMetrics(SM_CXSCREEN)-width)/2;
-			y = (GetSystemMetrics(SM_CYSCREEN)-height)/2;
+			if ( (flags & SDL_FULLSCREEN) ) {
+				x = (GetSystemMetrics(SM_CXSCREEN)-width)/2;
+				y = (GetSystemMetrics(SM_CYSCREEN)-height)/2;
+			} else if ( center ) {
+				x = (GetSystemMetrics(SM_CXSCREEN)-width)/2;
+				y = (GetSystemMetrics(SM_CYSCREEN)-height)/2;
+			} else if ( SDL_windowX || SDL_windowY || window ) {
+				x = bounds.left;
+				y = bounds.top;
+			} else {
+				x = y = -1;
+				swp_flags |= SWP_NOMOVE;
+			}
 			if ( y < 0 ) { /* Cover up title bar for more client area */
 				y -= GetSystemMetrics(SM_CYCAPTION)/2;
 			}
-			swp_flags = (SWP_NOCOPYBITS | SWP_FRAMECHANGED | SWP_SHOWWINDOW);
-			if ( was_visible && !(video->flags & SDL_FULLSCREEN) ) {
-				swp_flags |= SWP_NOMOVE;
-			}
-			if ( video->flags & SDL_FULLSCREEN ) {
+			if ( flags & SDL_FULLSCREEN ) {
 				top = HWND_TOPMOST;
 			} else {
 				top = HWND_NOTOPMOST;
@@ -1159,6 +1200,8 @@ SDL_Surface *DX5_SetVideoMode(_THIS, SDL_Surface *current,
 	}
 
 	/* Set the appropriate window style */
+	windowX = SDL_windowX;
+	windowY = SDL_windowY;
 	style = GetWindowLong(SDL_Window, GWL_STYLE);
 	style &= ~(resizestyle|WS_MAXIMIZE);
 	if ( (flags & SDL_FULLSCREEN) == SDL_FULLSCREEN ) {
@@ -1179,7 +1222,9 @@ SDL_Surface *DX5_SetVideoMode(_THIS, SDL_Surface *current,
 		if (IsZoomed(SDL_Window)) style |= WS_MAXIMIZE;
 #endif
 	}
-	SetWindowLong(SDL_Window, GWL_STYLE, style);
+	/* DJM: Don't piss of anyone who has setup his own window */
+	if ( SDL_windowid == NULL )
+		SetWindowLong(SDL_Window, GWL_STYLE, style);
 
 	/* Set DirectDraw sharing mode.. exclusive when fullscreen */
 	if ( (flags & SDL_FULLSCREEN) == SDL_FULLSCREEN ) {
@@ -1192,27 +1237,53 @@ SDL_Surface *DX5_SetVideoMode(_THIS, SDL_Surface *current,
 		SetDDerror("DirectDraw2::SetCooperativeLevel", result);
 		return(NULL);
 	}
+	SDL_windowX = windowX;
+	SDL_windowY = windowY;
 
 	/* Set the display mode, if we are in fullscreen mode */
 	if ( (flags & SDL_FULLSCREEN) == SDL_FULLSCREEN ) {
+		RECT bounds;
+		struct DX5EnumRect *rect;
+		int maxRefreshRate;
+
 		/* Cover up desktop during mode change */
 		SDL_resizing = 1;
-		SetWindowPos(SDL_Window, NULL, 0, 0, 
-			GetSystemMetrics(SM_CXSCREEN),
-			GetSystemMetrics(SM_CYSCREEN),
-			(SWP_NOCOPYBITS | SWP_NOZORDER));
+		bounds.left = 0;
+		bounds.top = 0;
+		bounds.right = GetSystemMetrics(SM_CXSCREEN);
+		bounds.bottom = GetSystemMetrics(SM_CYSCREEN);
+		AdjustWindowRectEx(&bounds, GetWindowLong(SDL_Window, GWL_STYLE), FALSE, 0);
+		SetWindowPos(SDL_Window, HWND_TOPMOST,
+			bounds.left, bounds.top, 
+			bounds.right - bounds.left,
+			bounds.bottom - bounds.top, SWP_NOCOPYBITS);
 		SDL_resizing = 0;
 		ShowWindow(SDL_Window, SW_SHOW);
 		while ( GetForegroundWindow() != SDL_Window ) {
 			SetForegroundWindow(SDL_Window);
 			SDL_Delay(100);
 		}
-		result = IDirectDraw2_SetDisplayMode(ddraw2, width, height,
-								bpp, 0, 0);
+
+		/* find maximum monitor refresh rate for this resolution */
+		/* Dmitry Yakimov ftech@tula.net */
+		maxRefreshRate = 0; /* system default */
+		for ( rect = enumlists[bpp / 8 - 1]; rect; rect = rect->next ) {
+			if ( (width == rect->r.w) && (height == rect->r.h) ) {
+				maxRefreshRate = rect->refreshRate;
+				break;
+			}
+		}
+#ifdef DDRAW_DEBUG
+ fprintf(stderr, "refresh rate = %d Hz\n", maxRefreshRate);
+#endif
+
+		result = IDirectDraw2_SetDisplayMode(ddraw2, width, height, bpp, maxRefreshRate, 0);
 		if ( result != DD_OK ) {
-			/* We couldn't set fullscreen mode, try window */
-			return(DX5_SetVideoMode(this, current,
-				width, height, bpp, flags & ~SDL_FULLSCREEN)); 
+			result = IDirectDraw2_SetDisplayMode(ddraw2, width, height, bpp, 0, 0);
+			if ( result != DD_OK ) {
+				/* We couldn't set fullscreen mode, try window */
+				return(DX5_SetVideoMode(this, current, width, height, bpp, flags & ~SDL_FULLSCREEN)); 
+			}
 		}
 		DX5_DInputReset(this, 1);
 	} else {
@@ -1252,11 +1323,7 @@ SDL_Surface *DX5_SetVideoMode(_THIS, SDL_Surface *current,
 		SetDDerror("DirectDrawSurface::QueryInterface", result);
 		return(NULL);
 	}
-	result = IDirectDrawSurface_Release(dd_surface1);
-	if ( result != DD_OK ) {
-		SetDDerror("DirectDrawSurface::Release", result);
-		return(NULL);
-	}
+	IDirectDrawSurface_Release(dd_surface1);
 
 	/* Get the format of the primary DirectDraw surface */
 	memset(&ddsd, 0, sizeof(ddsd));
@@ -1264,7 +1331,7 @@ SDL_Surface *DX5_SetVideoMode(_THIS, SDL_Surface *current,
 	ddsd.dwFlags = DDSD_PIXELFORMAT|DDSD_CAPS;
 	result = IDirectDrawSurface3_GetSurfaceDesc(SDL_primary, &ddsd);
 	if ( result != DD_OK ) {
-		SetDDerror("DirectDrawSurface::Release", result);
+		SetDDerror("DirectDrawSurface::GetSurfaceDesc", result);
 		return(NULL);
 	}
 	if ( ! (ddsd.ddpfPixelFormat.dwFlags&DDPF_RGB) ) {
@@ -1449,6 +1516,8 @@ SDL_Surface *DX5_SetVideoMode(_THIS, SDL_Surface *current,
 		RECT bounds;
 		int  x, y;
 		UINT swp_flags;
+		const char *window = getenv("SDL_VIDEO_WINDOW_POS");
+		const char *center = getenv("SDL_VIDEO_CENTERED");
 
 		/* Create and set a clipper on our primary surface */
 		if ( SDL_clipper == NULL ) {
@@ -1480,26 +1549,47 @@ SDL_Surface *DX5_SetVideoMode(_THIS, SDL_Surface *current,
 			return(NULL);
 		}
 
-		/* Set the size of the window, centering and adjusting */
+		if ( !SDL_windowX && !SDL_windowY ) {
+			if ( window ) {
+				if ( sscanf(window, "%d,%d", &x, &y) == 2 ) {
+					SDL_windowX = x;
+					SDL_windowY = y;
+				}
+				if ( strcmp(window, "center") == 0 ) {
+					center = window;
+					window = NULL;
+				}
+			}
+		}
+		swp_flags = SWP_NOCOPYBITS;
+
 		SDL_resizing = 1;
-		bounds.top    = 0;
-		bounds.bottom = video->h;
-		bounds.left   = 0;
-		bounds.right  = video->w;
+		bounds.left = SDL_windowX;
+		bounds.top = SDL_windowY;
+		bounds.right = SDL_windowX+video->w;
+		bounds.bottom = SDL_windowY+video->h;
 		AdjustWindowRectEx(&bounds, GetWindowLong(SDL_Window, GWL_STYLE), FALSE, 0);
 		width = bounds.right-bounds.left;
 		height = bounds.bottom-bounds.top;
-		x = (GetSystemMetrics(SM_CXSCREEN)-width)/2;
-		y = (GetSystemMetrics(SM_CYSCREEN)-height)/2;
+		if ( (flags & SDL_FULLSCREEN) ) {
+			x = (GetSystemMetrics(SM_CXSCREEN)-width)/2;
+			y = (GetSystemMetrics(SM_CYSCREEN)-height)/2;
+		} else if ( center ) {
+			x = (GetSystemMetrics(SM_CXSCREEN)-width)/2;
+			y = (GetSystemMetrics(SM_CYSCREEN)-height)/2;
+		} else if ( SDL_windowX || SDL_windowY || window ) {
+			x = bounds.left;
+			y = bounds.top;
+		} else {
+			x = y = -1;
+			swp_flags |= SWP_NOMOVE;
+		}
 		if ( y < 0 ) { /* Cover up title bar for more client area */
 			y -= GetSystemMetrics(SM_CYCAPTION)/2;
 		}
-		swp_flags = (SWP_NOCOPYBITS | SWP_NOZORDER);
-		if ( was_visible ) {
-			swp_flags |= SWP_NOMOVE;
-		}
-		SetWindowPos(SDL_Window, NULL, x, y, width, height, swp_flags);
+		SetWindowPos(SDL_Window, HWND_NOTOPMOST, x, y, width, height, swp_flags);
 		SDL_resizing = 0;
+
 	}
 	ShowWindow(SDL_Window, SW_SHOW);
 	SetForegroundWindow(SDL_Window);
@@ -1953,11 +2043,17 @@ static int DX5_FlipHWSurface(_THIS, SDL_Surface *surface)
 	LPDIRECTDRAWSURFACE3 dd_surface;
 
 	dd_surface = surface->hwdata->dd_surface;
-	result = IDirectDrawSurface3_Flip(dd_surface, NULL, 0);
+
+	/* to prevent big slowdown on fast computers, wait here instead of driver ring 0 code */
+	/* Dmitry Yakimov (ftech@tula.net) */
+	while(IDirectDrawSurface3_GetFlipStatus(dd_surface, DDGBS_ISBLTDONE) == DDERR_WASSTILLDRAWING);
+
+	result = IDirectDrawSurface3_Flip(dd_surface, NULL, DDFLIP_WAIT);
 	if ( result == DDERR_SURFACELOST ) {
 		result = IDirectDrawSurface3_Restore(
 						surface->hwdata->dd_surface);
-		result = IDirectDrawSurface3_Flip(dd_surface, NULL, 0);
+		while(IDirectDrawSurface3_GetFlipStatus(dd_surface, DDGBS_ISBLTDONE) == DDERR_WASSTILLDRAWING);
+		result = IDirectDrawSurface3_Flip(dd_surface, NULL, DDFLIP_WAIT);
 	}
 	if ( result != DD_OK ) {
 		SetDDerror("DirectDrawSurface3::Flip", result);
