@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
+    Copyright (C) 1997-2004 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_x11mouse.c,v 1.8 2002/10/11 08:09:26 slouken Exp $";
+ "@(#) $Id: SDL_x11mouse.c,v 1.10 2004/05/16 17:40:32 slouken Exp $";
 #endif
 
 #include <stdlib.h>
@@ -142,6 +142,8 @@ WMcursor *X11_CreateWMCursor(_THIS,
 	/* Create the cursor */
 	cursor->x_cursor = XCreatePixmapCursor(SDL_Display, data_pixmap,
 				mask_pixmap, &black, &white, hot_x, hot_y);
+	XFreePixmap(SDL_Display, data_pixmap);
+	XFreePixmap(SDL_Display, mask_pixmap);
 
 	/* Release the event thread */
 	XSync(SDL_Display, False);
